import { mapModelSchema } from '@/models/modelSchema';
import { apiPath } from '@/redux/apiPath';
import { axiosInstance } from '@/services/api/utils/axiosInstance';
import { MapModel } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { z } from 'zod';
import { ThunkConfig } from '@/types/store';
import { getError } from '@/utils/error-report/getError';
import { MODELS_FETCHING_ERROR_PREFIX } from './models.constants';

export const getModels = createAsyncThunk<MapModel[] | undefined, void, ThunkConfig>(
  'project/getModels',
  async () => {
    try {
      const response = await axiosInstance.get<MapModel[]>(apiPath.getModelsString());

      const isDataValid = validateDataUsingZodSchema(response.data, z.array(mapModelSchema));

      return isDataValid ? response.data : undefined;
    } catch (error) {
      return Promise.reject(getError({ error, prefix: MODELS_FETCHING_ERROR_PREFIX }));
    }
  },
);
