/* eslint-disable no-magic-numbers */
import { reactionsFixture } from '@/models/fixtures/reactionFixture';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import { apiPath } from '../apiPath';
import reactionsReducer from './reactions.slice';
import { getReactionsByIds } from './reactions.thunks';
import { ReactionsState } from './reactions.types';

const mockedAxiosClient = mockNetworkResponse();

describe('reactions thunks', () => {
  let store = {} as ToolkitStoreWithSingleSlice<ReactionsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('reactions', reactionsReducer);
  });

  describe('getReactionsByIds', () => {
    it('should return data when data response from API is valid', async () => {
      const ids = [1];

      mockedAxiosClient
        .onGet(apiPath.getReactionsWithIds(ids))
        .reply(HttpStatusCode.Ok, reactionsFixture);

      const { payload } = await store.dispatch(getReactionsByIds(ids));
      expect(payload).toEqual({ data: reactionsFixture, shouldConcat: false });
    });

    it('should return undefined when data response from API is not valid ', async () => {
      mockedAxiosClient
        .onGet(apiPath.getReactionsWithIds([]))
        .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

      const { payload } = await store.dispatch(getReactionsByIds([]));
      expect(payload).toEqual(undefined);
    });

    it('should return empty array when data response from API is empty', async () => {
      mockedAxiosClient.onGet(apiPath.getReactionsWithIds([100])).reply(HttpStatusCode.Ok, []);

      const { payload } = await store.dispatch(getReactionsByIds([100]));
      expect(payload).toEqual({ data: [], shouldConcat: false });
    });
  });
});
